//
//  CompassViewController.m
//  Compass
//
//  Created by Alasdair Allan on 18/05/2011.
//  Copyright 2011 University of Exeter. All rights reserved.
//

#import "CompassViewController.h"

@implementation CompassViewController

- (void)dealloc
{
    [arrowImage release];
    [magneticHeadingLabel release];
    [trueHeadingLabel release];
    [orientationLabel release];
    [super dealloc];
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (void)viewDidLoad {
    [super viewDidLoad];
	CLLocationManager *locationManager = [[CLLocationManager alloc] init];
	locationManager.delegate = self;
	if( [CLLocationManager locationServicesEnabled] &&  [CLLocationManager headingAvailable])
	{
		[locationManager startUpdatingLocation];
		[locationManager startUpdatingHeading];
	} else {
		NSLog(@"Can't report heading");
	}
}

- (void)viewDidUnload
{
    [arrowImage release];
    arrowImage = nil;
    [magneticHeadingLabel release];
    magneticHeadingLabel = nil;
    [trueHeadingLabel release];
    trueHeadingLabel = nil;
    [orientationLabel release];
    orientationLabel = nil;
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

- (void)locationManager:(CLLocationManager*)manager didUpdateHeading:(CLHeading*)newHeading {
    
	UIDevice *device = [UIDevice currentDevice];
    orientationLabel.text = [self stringFromOrientation:device.orientation];
	
	if (newHeading.headingAccuracy > 0) {
		float magneticHeading = [self magneticHeading:newHeading.magneticHeading fromOrientation:device.orientation];
		float trueHeading = [self trueHeading:newHeading.trueHeading fromOrientation:device.orientation];
		
		magneticHeadingLabel.text = [NSString stringWithFormat:@"%f", magneticHeading];
		trueHeadingLabel.text = [NSString stringWithFormat:@"%f", trueHeading];
		
		float heading = -1.0f * M_PI * newHeading.magneticHeading / 180.0f;
		arrowImage.transform = CGAffineTransformMakeRotation(heading);
	}

}

-(float)magneticHeading:(float)heading fromOrientation:(UIDeviceOrientation) orientation {
	
	float realHeading = heading;
	switch (orientation) {
		case UIDeviceOrientationPortrait: 
			break;
		case UIDeviceOrientationPortraitUpsideDown: 
			realHeading = realHeading + 180.0f; 
			break;
		case UIDeviceOrientationLandscapeLeft: 
			realHeading = realHeading + 90.0f; 
			break;
		case UIDeviceOrientationLandscapeRight: 
			realHeading = realHeading - 90.0f; 
			break;
		default: 
			break;
	}
	while ( realHeading > 360.0f ) {
		realHeading = realHeading - 360;
	}
	return realHeading;
}

-(float)trueHeading:(float)heading fromOrientation:(UIDeviceOrientation) orientation {
	
	float realHeading = heading;
	switch (orientation) {
		case UIDeviceOrientationPortrait: 
			break;
		case UIDeviceOrientationPortraitUpsideDown: 
			realHeading = realHeading + 180.0f; 
			break;
		case UIDeviceOrientationLandscapeLeft: 
			realHeading = realHeading + 90.0f; 
			break;
		case UIDeviceOrientationLandscapeRight: 
			realHeading = realHeading - 90.0f; 
			break;
		default: 
			break;
	}
	while ( realHeading > 360.0f ) {
		realHeading = realHeading - 360;
	}
	return realHeading;
}

- (NSString *)stringFromOrientation:(UIDeviceOrientation) orientation {
	
    NSString *orientationString;
	switch (orientation) {
		case UIDeviceOrientationPortrait: 
			orientationString =  @"Portrait"; 
			break;
		case UIDeviceOrientationPortraitUpsideDown: 
			orientationString =  @"Portrait Upside Down"; 
			break;
		case UIDeviceOrientationLandscapeLeft: 
			orientationString =  @"Landscape Left"; 
			break;
		case UIDeviceOrientationLandscapeRight: 
			orientationString =  @"Landscape Right"; 
			break;
		case UIDeviceOrientationFaceUp: 
			orientationString =  @"Face Up"; 
			break;
		case UIDeviceOrientationFaceDown: 
			orientationString =  @"Face Down"; 
			break;
		case UIDeviceOrientationUnknown: 
			orientationString = @"Unknown";
			break;
		default: 
			orientationString = @"Not Known";
			break;
	}
	return orientationString;
}

@end
