//
//  ViewController.m
//  GeoCoder
//
//  Created by Alasdair Allan on 02/09/2012.
//  Copyright (c) 2012 Babilim Light Industries. All rights reserved.
//

#import "ViewController.h"
#import <CoreLocation/CoreLocation.h>

@interface ViewController ()

@end

@implementation ViewController
@synthesize latitude;
@synthesize longitude;
@synthesize address;

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.
}

- (void)viewDidUnload
{
    [self setLatitude:nil];
    [self setLongitude:nil];
    [self setAddress:nil];
    [super viewDidUnload];
    // Release any retained subviews of the main view.
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return (interfaceOrientation != UIInterfaceOrientationPortraitUpsideDown);
}

- (IBAction)reverseButton:(id)sender {
    
    CLLocation *location = [[CLLocation alloc] initWithLatitude:[self.latitude.text doubleValue] longitude:[self.longitude.text doubleValue]];
    
    NSLog(@"location = %@", location);
    CLGeocoder *geocoder = [[CLGeocoder alloc] init];
    
    [geocoder reverseGeocodeLocation: location
                   completionHandler: ^(NSArray *placemarks, NSError *error) {
                       
                       NSLog(@"completed");
                       if ( error ) {
                           NSLog(@"error = %@", error );
                           dispatch_async(dispatch_get_main_queue(),^{
                                                              
                               UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Error" message:[self errorMessage:error.code]  delegate:nil cancelButtonTitle:nil  otherButtonTitles:@"OK", nil];
                               [alert show];
                               
                           });
                           
                       }
                       for (CLPlacemark *placemark in placemarks) {
                           
                           NSLog(@"Placemark is %@", placemark);
                           UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Result" message:[NSString stringWithFormat:@"%@", placemark] delegate:nil cancelButtonTitle:nil otherButtonTitles:@"OK", nil];
                           [alert show];
                           
                       }
                   }];
    
    
}

- (IBAction)forwardButton:(id)sender {

    NSString *place = self.address.text;
    CLGeocoder *geocoder = [[CLGeocoder alloc] init];
    
    [geocoder geocodeAddressString:place completionHandler:^(NSArray* placemarks, NSError* error){
        
        NSLog(@"completed");
        if ( error ) {
            NSLog(@"error = %@", error );
            dispatch_async(dispatch_get_main_queue(),^{
                
                UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Error" message:[self errorMessage:error.code]  delegate:nil cancelButtonTitle:nil  otherButtonTitles:@"OK", nil];
                [alert show];
                
            });
            
        }
        for (CLPlacemark *placemark in placemarks) {
            
            NSLog(@"Placemark is %@", placemark);
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Result" message:[NSString stringWithFormat:@"%@", placemark] delegate:nil cancelButtonTitle:nil otherButtonTitles:@"OK", nil];
            [alert show];
            
        }
    }];
    
}


- (NSString *)errorMessage:(int) code {
    
    NSString *message = nil;
    switch (code) {
        case kCLErrorLocationUnknown:
            message = @"Location Unknown";
            break;
        case kCLErrorDenied:
            message = @"Denied";
            break;
        case kCLErrorNetwork:
            message = @"Network Error";
            break;
        case kCLErrorGeocodeFoundNoResult:
            message = @"No Result Found";
            break;
        case kCLErrorGeocodeFoundPartialResult:
            message = @"Partial Result";
            break;
        case kCLErrorGeocodeCanceled:
            message = @"Cancelled";
            break;
        default:
            message = @"Unknown Error";
            break;
    }
    return message;
    
}


#pragma mark - UITextField

- (BOOL)textFieldShouldBeginEditing:(UITextField *)textField {
    activeTextField = textField;
    
    CGAffineTransform transform;
    switch (textField.tag) {
        case 1:
            transform = CGAffineTransformMakeTranslation(0,0);
            break;
        case 2:
            transform = CGAffineTransformMakeTranslation(0,-50);
            break;
        case 3:
            transform = CGAffineTransformMakeTranslation(0,-120);
            break;
        default:
            transform = CGAffineTransformMakeTranslation(0,0);
            break;
    }
    [UIView beginAnimations:@"MoveAnimation" context:nil];
    [UIView setAnimationCurve:UIViewAnimationCurveEaseInOut];
    [UIView setAnimationDuration:0.3];
    self.view.transform = transform;
    [UIView commitAnimations];
    
    return YES;
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField {
    activeTextField = nil;
    
    CGAffineTransform transform = CGAffineTransformMakeTranslation(0,0);
    [UIView beginAnimations:@"MoveAnimation" context:nil];
    [UIView setAnimationCurve:UIViewAnimationCurveEaseInOut];
    [UIView setAnimationDuration:0.3];
    self.view.transform = transform;
    [UIView commitAnimations];

    [textField resignFirstResponder];
    return YES;
}

@end

