//
//  MapOverlayView.m
//  HeatMap
//
//  Created by Alasdair Allan on 01/09/2012.
//  Copyright (c) 2012 Babilim Light Industries. All rights reserved.
//

#import "MapOverlayView.h"
#import "MapOverlay.h"
#import "SHGeoUtils.h"

@implementation MapOverlayView

- (void)drawMapRect:(MKMapRect)mapRect zoomScale:(MKZoomScale)zoomScale inContext:(CGContextRef)ctx {
    
    // Load in earth quake data
    NSString *dataFile = [[NSBundle mainBundle] pathForResource:@"EarthQuake-Data" ofType:@"plist"];
    NSArray *quakeData = [[NSArray alloc] initWithContentsOfFile:dataFile];
    
    NSMutableArray *locations = [[NSMutableArray alloc] initWithCapacity:[quakeData count]];
    NSMutableArray *weights = [[NSMutableArray alloc] initWithCapacity:[quakeData count]];

    for (NSDictionary *reading in quakeData) {
        CLLocationDegrees latitude = [[reading objectForKey:@"latitude"] doubleValue];
        CLLocationDegrees longitude = [[reading objectForKey:@"longitude"] doubleValue];
        double magnitude = [[reading objectForKey:@"magnitude"] doubleValue];
        
        CLLocation *location = [[CLLocation alloc] initWithLatitude:latitude longitude:longitude];
        [locations addObject:location];
        [weights addObject:[NSNumber numberWithInteger:(magnitude * 10)]];
    }
    
    //Loading and setting the image
    MKMapRect theMapRect = [self.overlay boundingMapRect];
    CGRect theCGRect = [self rectForMapRect:theMapRect];

    MapOverlay *overlay = (MapOverlay *)self.overlay;
    MKMapView *view = overlay.mapView;
    
    UIImage *heatmap = [SHGeoUtils heatMapForMapView:view boost:0.67 locations:locations weights:weights];
    CGImageRef imageRef = heatmap.CGImage;
    
    // We need to flip and reposition the image here
    CGContextScaleCTM(ctx, 1.0, -1.0);
    CGContextTranslateCTM(ctx, 0.0, -theCGRect.size.height);
    
    //drawing the image to the context
    CGContextDrawImage(ctx, theCGRect, imageRef);
}


@end
