//
//  ViewController.m
//  Location
//
//  Created by Alasdair Allan on 29/01/2012.
//  Copyright (c) 2012 University of Exeter. All rights reserved.
//

#import "ViewController.h"
#import "AppDelegate.h"

@implementation ViewController

@synthesize tableView = _tableView;
@synthesize mapCell = _mapCell;
@synthesize mapView = _mapView;

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

#pragma mark - View lifecycle

- (void)viewDidLoad {
    self.mapView.layer.cornerRadius = 10.0;
    self.mapView.showsUserLocation = YES;
    
    CLLocationCoordinate2D moffett = {37.4163, -122.0519};
    SimpleAnnotation *moffettAnnotation =
    [[SimpleAnnotation alloc] initWithCoordinate:moffett];
    moffettAnnotation.title = @"Moffett Federal Airfield";
    moffettAnnotation.subtitle = @"37.4163, -122.0519";
    [self.mapView addAnnotation: moffettAnnotation];
    
    CLLocationCoordinate2D sanJose = {37.3647, -121.9338};
    SimpleAnnotation *sanJoseAnnotation =
    [[SimpleAnnotation alloc] initWithCoordinate:sanJose];
    sanJoseAnnotation.title = @"San Jose International";
    sanJoseAnnotation.subtitle = @"37.3647, -121.9338";
    [self.mapView addAnnotation: sanJoseAnnotation];
    
    [super viewDidLoad];
}

- (void)viewDidUnload {
    [self setMapCell:nil];
    [self setMapView:nil];
    [super viewDidUnload];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
}

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
    
    CLLocationCoordinate2D center = CLLocationCoordinate2DMake(37.3647, -121.9338);
    CLLocationDistance radius = 3000.0;
    MKCircle *circle = [MKCircle circleWithCenterCoordinate:center radius:radius];
    [circle setTitle:@"San Jose Airport"];
    [self.mapView addOverlay:circle];
}

- (void)viewWillDisappear:(BOOL)animated {
	[super viewWillDisappear:animated];
}

- (void)viewDidDisappear:(BOOL)animated {
	[super viewDidDisappear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return (interfaceOrientation != UIInterfaceOrientationPortraitUpsideDown);
}

#pragma mark UITableViewDelegate Methods

- (void)tableView:(UITableView *)tv didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    // Add code here
}

#pragma mark UITableViewDataSource Methods

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    NSInteger height;
    if ( indexPath.section == 0 ) {
        height = 170;
    } else {
        height = 44;
    }
    return height;
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tv {
    return 3;
}

- (NSInteger) tableView:(UITableView *)tv numberOfRowsInSection:(NSInteger) section {
    if ( section == 0 ) {
        return 1;
    } else if ( section == 1 ) {
        return 5;
    } else {
        return 1;
    }
}

- (UITableViewCell *)tableView:(UITableView *)tv cellForRowAtIndexPath:(NSIndexPath *)indexPath  {
    
    UITableViewCell *cell = nil;
    if ( indexPath.section == 0 ) {
        cell = self.mapCell;
        
    } else {
        static NSString *identifier = @"cell";
        cell = [tv dequeueReusableCellWithIdentifier:@"cell"];

        if ( cell == nil ) {
            cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:identifier];
            cell.accessoryType = UITableViewCellAccessoryNone;
        }
        AppDelegate *delegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
        if ( indexPath.section == 1 ) {
            cell.textLabel.text = [delegate.rows objectAtIndex:indexPath.row];
        } else {
            cell.textLabel.text = [NSString stringWithFormat:@"Positions %d", [delegate getSizeOfDatabase]];
        }
        
    }
    return cell;
    
}


- (MKOverlayView *)mapView:(MKMapView *)mapView viewForOverlay:(id <MKOverlay>)overlay {
    
    NSLog(@"Overlaying circle");
    MKCircle *circle = overlay;
    MKCircleView *circleView = [[MKCircleView alloc] initWithCircle:overlay];
    
    if ([circle.title isEqualToString:@"San Jose Airport"]) {
         circleView.fillColor = [UIColor redColor];
        circleView.alpha = 0.5;
    }
    return circleView;
}


@end
